%% Plot the graphs in Matlab
addpath('function');
close all; 

time = out.q.Time; % Time axis

q = out.q.Data; % Actual joint angles
q1 = q(1,:); q2 = q(2,:); q3 = q(3,:);

qd = out.qd.Data; % Desired joint angles
qd1 = qd(1,:); qd2 = qd(2,:); qd3 = qd(3,:);

dq = out.dq.Data; % Actual joint velocity
dq1 = dq(1,:);  dq2 = dq(2,:);  dq3 = dq(3,:);

dqd = out.dqd.Data; % Desired joint velocity
dqd1 = dqd(1,:);  dqd2 = dqd(2,:);  dqd3 = dqd(3,:);

torque = out.joint_torque.Data; % Joint torque
torque1 = torque(1,:); torque2 = torque(2,:); torque3 = torque(3,:);

%% Plot the joint angle
figure('Name', 'Joint angle','NumberTitle','off');
% Axis scaling
x_min = min(time); x_max = max(time); 
yMax = max([max(q1), max(q2), max(q3)]);
yMin = min([min(q1), min(q2), min(q3)]);
y_min = yMin - (yMax - yMin)*0.1;
y_max = yMax + (yMax - yMin)*0.1;
axis([x_min x_max y_min y_max]);

plot(time, qd1, 'k--', time, q1, 'r', time, qd2, 'k--',time, q2, 'b', time, qd3, 'k--', time, q3, 'g');
xlabel('Time (s)');  ylabel('Joint angle (deg)');
legend('Joint 1', 'Joint 1', 'Joint 2', 'Joint 2', 'Joint 3', 'Joint 3');  
grid on;

%% Plot the joint velocity
figure('Name', 'Joint velocity','NumberTitle','off');
% Axis scaling
yMax = max([max(dq1), max(dq2), max(dq3)]);
yMin = min([min(dq1), min(dq2), min(dq3)]);
y_min = yMin - (yMax - yMin)*0.1;
y_max = yMax + (yMax - yMin)*0.1;
axis([x_min x_max y_min y_max]);

plot(time, dqd1, 'k--', time, dq1, 'r', time, dqd2, 'k--',time, dq2, 'b', time, dqd3, 'k--', time, dq3, 'g');
xlabel('Time (s)');  ylabel('Joint Velocity (deg/s)');
legend('Joint 1', 'Joint 1', 'Joint 2', 'Joint 2', 'Joint 3', 'Joint 3');  
grid on;

%% Plot the joint torque
figure('Name', 'Joint torque','NumberTitle','off');
% Axis scaling
yMax = max([max(torque1), max(torque2), max(torque3)]);
yMin = min([min(torque1), min(torque2), min(torque3)]);
y_min = yMin - (yMax-yMin)*0.1; y_max = yMax + (yMax-yMin)*0.1;
axis([x_min x_max y_min y_max]);

plot(time, torque1, 'r', time, torque2, 'b', time, torque3, 'g');
xlabel('Time (s)');  ylabel('Joint torque (Nm)');
legend('Joint 1', 'Joint 2', 'Joint 3');  grid on;

