%% Set Parameters of a 3 DOF Spatial Arm
addpath('function');
clear; clc;

%% Variables
% Trajectory
ini_q = [0, 0, 0]; fin_q = [90, 60, 45]; % unit: deg
end_time = [1; 1; 1];
waypoint = [ini_q'*pi/180, fin_q'*pi/180];
ini_q1 = ini_q(1)*pi/180; ini_q2 = ini_q(2)*pi/180; ini_q3 = ini_q(3)*pi/180;
fin_q1 = fin_q(1)*pi/180; fin_q2 = fin_q(2)*pi/180; fin_q3 = fin_q(3)*pi/180;

% Link parameters
Mass   = [9.4, 9.9, 8.2];       % unit: kg
Length = [0.320, 0.370, 0.420]; % unit: m

% DH parameters
global a1 a2 a3 d1 d2 d3 m1 m2 m3;
a1 = 0; a2 = Length(2); a3 = Length(3); % Link length
d1 = Length(1); d2 = 0; d3 = 0; % Joint offset
m1 = Mass(1); m2 = Mass(2); m3 = Mass(3); % Link mass

% Vector from origin of frame {i} to COM of frame {i} w.r.t frame {i}
global r1cx r1cy r1cz r2cx r2cy r2cz r3cx r3cy r3cz;
r1cx = 0; r1cy = -Length(1)/2; r1cz = 0;
r2cx = -Length(2)/2; r2cy = 0; r2cz = 0;
r3cx = -Length(3)/2; r3cy = 0; r3cz = 0;

% Inertia of link 1
global J1xx J1xy J1xz J1yx J1yy J1yz J1zx J1zy J1zz;
J1xx = m1*Length(1)^2/3; J1xy = 0;  J1xz = 0;
J1yx = 0; J1yy = 0; J1yz = 0;
J1zx = 0; J1zy = 0; J1zz = m1*Length(1)^2/3;

% Inertia of link 2
global J2xx J2xy J2xz J2yx J2yy J2yz J2zx J2zy J2zz; 
J2xx = 0; J2xy = 0; J2xz = 0;
J2yx = 0; J2yy = m2*Length(2)^2/3; J2yz = 0;
J2zx = 0; J2zy = 0; J2zz = m2*Length(2)^2/3;

% Inertia of link 3
global J3xx J3xy J3xz J3yx J3yy J3yz J3zx J3zy J3zz;
J3xx = 0; J3xy = 0; J3xz = 0;
J3yx = 0; J3yy = m3*Length(3)^2/3; J3yz = 0;
J3zx = 0; J3zy = 0; J3zz = m3*Length(3)^2/3;
