%% MATLAB code: step1_compute_torques.m (for 3 DOF spatial arm) 
clear; clc
 
% Declaration of symbolic variables
syms m1 m2 m3;  % Link mass
syms a1 a2 a3;  % Link length
syms d1 d2 d3;  % Joint offset
syms g;  % Acceleration of gravity
syms c1 c2 c3;  % Cosine function
syms s1 s2 s3;  % Sine function
syms q1 q2 q3;  % Joint angle
syms dq1 dq2 dq3;  % Joint velocity
syms ddq1 ddq2 ddq3;  % Joint acceleration
 
% Position vectors to link center of mass 
syms r1cx r1cy r1cz;
syms r2cx r2cy r2cz;
syms r3cx r3cy r3cz;
r1c = [r1cx; r1cy; r1cz];
r2c = [r2cx; r2cy; r2cz];
r3c = [r3cx; r3cy; r3cz];
 
% Inertia matrices
syms J1xx J1xy J1xz J1yx J1yy J1yz J1zx J1zy J1zz;
syms J2xx J2xy J2xz J2yx J2yy J2yz J2zx J2zy J2zz;
syms J3xx J3xy J3xz J3yx J3yy J3yz J3zx J3zy J3zz;
J1 = [J1xx J1xy J1xz; J1xy J1yy J1yz; J1xz J1yz J1zz];
J2 = [J2xx J2xy J2xz; J2xy J2yy J2yz; J2xz J2yz J2zz];
J3 = [J3xx J3xy J3xz; J3xy J3yy J3yz; J3xz J3yz J3zz];

% Rotation matrices of PUMA robot
R01 = [c1 0 s1; s1 0 -c1; 0 1 0];     
R12 = [c2 -s2 0; s2 c2 0; 0 0 1];
R23 = [c3 -s3 0; s3 c3 0; 0 0 1];
R34 = eye(3);
R02 = R01*R12;
R03 = R02*R23;

% Position vector from frame {i-1} to frame {i} w.r.t. frame {i}
r01 = [0;d1;0];
r12 = [a2;0;0];
r23 = [a3;0;0];

% Gravity and z0 vectors
G = [0; 0; -g];  % Gravity vector
z0 = [0; 0; 1];  % z0 vector

% Initial conditions
w00 = [0; 0; 0];  % Angular velocity
dw00 = [0; 0; 0];  % Angular acceleration
dv00 = [0; 0; 0];  % Linear acceleration

% Terminal conditions
f44 = [0;0;0]; % Force applied to environment
n44 = [0;0;0]; % Moment applied to environment

%% Forward recursion
% Link 1
w11 = w(R01,w00,dq1,z0);
dw11 = dw(R01,w00,dw00,dq1,ddq1,z0);
dv11 = dv(R01,w11,dw11,dv00,r01);
dv1c1 = dvc(dv11,w11,dw11,r1c);
'Forward recursion for link 1 completed'
 
% Link 2
w22 = w(R12,w11,dq2,z0);
dw22 = dw(R12,w11,dw11,dq2,ddq2,z0);
dv22 = dv(R12,w22,dw22,dv11,r12);
dv2c2 = dvc(dv22,w22,dw22,r2c);
'Forward recursion for link 2 completed'
 
% Link 3
w33 = w(R23,w22,dq3,z0);
dw33 = dw(R23,w22,dw22,dq3,ddq3,z0);
dv33 = dv(R23,w33,dw33,dv22,r23);
dv3c3 = dvc(dv33,w33,dw33,r3c);
'Forward recursion for link 3 completed'
 
%% Backward recursion
% Link 3
f33 = f(R34,f44,m3,R03,G,dv3c3);
n33 = n(R34,J3,f33,f44,r23,r3c,w33,dw33,n44);
tau3 = tau(R23,n33,z0);
'Backward recursion for link 3 completed'
 
% Link 2
f22 = f(R23,f33,m2,R02,G,dv2c2);
n22 = n(R23,J2,f22,f33,r12,r2c,w22,dw22,n33);
tau2 = tau(R12,n22,z0);
'Backward recursion for link 2 completed'
 
% Link 1
f11 = f(R12,f22,m1,R01,G,dv1c1);
n11 = n(R12,J1,f11,f22,r01,r1c,w11,dw11,n22);
tau1 = tau(R01,n11,z0);
'Backward recursion for link 1 completed'
 
'Step 1 completed'
