%% MATLAB code: step1_compute_torques.m (for PUMA robot) 
clear; clc
 
% Declaration of symbolic variables
syms m1 m2 m3 m4 m5 m6;  % Link mass
syms a1 a2 a3 a4 a5 a6;  % Link length
syms d1 d2 d3 d4 d5 d6;  % Joint offset
syms g;  % Acceleration of gravity
syms c1 c2 c3 c4 c5 c6;  % Cosine function
syms s1 s2 s3 s4 s5 s6;  % Sine function
syms q1 q2 q3 q4 q5 q6;  % Joint angle
syms dq1 dq2 dq3 dq4 dq5 dq6;  % Joint velocity
syms ddq1 ddq2 ddq3 ddq4 ddq5 ddq6;  % Joint acceleration

% Position vectors to the center of mass of link 
syms r1cx r1cy r1cz r2cx r2cy r2cz r3cx r3cy r3cz;
syms r4cx r4cy r4cz r5cx r5cy r5cz r6cx r6cy r6cz;
r1c = [r1cx; r1cy; r1cz]; r2c = [r2cx; r2cy; r2cz]; r3c = [r3cx; r3cy; r3cz];
r4c = [r4cx; r4cy; r4cz]; r5c = [r5cx; r5cy; r5cz]; r6c = [r6cx; r6cy; r6cz];
 
% Inertia matrices
syms J1xx J1xy J1xz J1yx J1yy J1yz J1zx J1zy J1zz;
syms J2xx J2xy J2xz J2yx J2yy J2yz J2zx J2zy J2zz;
syms J3xx J3xy J3xz J3yx J3yy J3yz J3zx J3zy J3zz;
syms J4xx J4xy J4xz J4yx J4yy J4yz J4zx J4zy J4zz;
syms J5xx J5xy J5xz J5yx J5yy J5yz J5zx J5zy J5zz;
syms J6xx J6xy J6xz J6yx J6yy J6yz J6zx J6zy J6zz;
J1 = [J1xx J1xy J1xz; J1xy J1yy J1yz; J1xz J1yz J1zz];
J2 = [J2xx J2xy J2xz; J2xy J2yy J2yz; J2xz J2yz J2zz];
J3 = [J3xx J3xy J3xz; J3xy J3yy J3yz; J3xz J3yz J3zz];
J4 = [J4xx J4xy J4xz; J4xy J4yy J4yz; J4xz J4yz J4zz];
J5 = [J5xx J5xy J5xz; J5xy J5yy J5yz; J5xz J5yz J5zz];
J6 = [J6xx J6xy J6xz; J6xy J6yy J6yz; J6xz J6yz J6zz];

% Rotation matrices of PUMA robot
R01 = [c1 0 s1; s1 0 -c1; 0 1 0];
R12 = [c2 -s2 0; s2 c2 0; 0 0 1];
R23 = [c3 0 s3; s3 0 -c3; 0 1 0];
R34 = [c4 0 -s4; s4 0 c4; 0 -1 0];
R45 = [c5 0 s5; s5 0 -c5; 0 1 0];
R56 = [c6 -s6 0; s6 c6 0; 0 0 1];
R67 = eye(3);
R02 = R01*R12;  R03 = R02*R23;
R04 = R03*R34;  R05 = R04*R45;  R06 = R05*R56;

% Position vector from frame {i-1} to frame {i} w.r.t. frame {i}
r01 = [0; d1; 0]; r12 = [a2; 0; 0]; r23 = [0; 0; 0];
r34 = [0; -d4; 0]; r45 = [0; 0; 0]; r56 = [0; 0; d6];

% Gravity and z0 vectors
G = [0; 0; -g];  % Gravity vector
z0 = [0; 0; 1];  % z0 vector

% Initial conditions
w00 = [0; 0; 0];  % Angular velocity
dw00 = [0; 0; 0];  % Angular acceleration
dv00 = [0; 0; 0];  % Linear acceleration

% Terminal conditions
f77 = [0;0;0]; % Force applied to the environment
n77 = [0;0;0]; % Moment applied to the environment

%% Forward recursion
% Link 1
w11 = w(R01,w00,dq1,z0);
dw11 = dw(R01,w00,dw00,dq1,ddq1,z0);
dv11 = dv(R01,w11,dw11,dv00,r01);
dv1c1 = dvc(dv11,w11,dw11,r1c);
'Forward recursion for link 1 completed'
 
% Link 2
w22 = w(R12,w11,dq2,z0);
dw22 = dw(R12,w11,dw11,dq2,ddq2,z0);
dv22 = dv(R12,w22,dw22,dv11,r12);
dv2c2 = dvc(dv22,w22,dw22,r2c);
'Forward recursion for link 2 completed'
 
% Link 3
w33 = w(R23,w22,dq3,z0);
dw33 = dw(R23,w22,dw22,dq3,ddq3,z0);
dv33 = dv(R23,w33,dw33,dv22,r23);
dv3c3 = dvc(dv33,w33,dw33,r3c);
'Forward recursion for link 3 completed'
 
% Link 4
w44 = w(R34,w33,dq3,z0);
dw44 = dw(R34,w33,dw33,dq4,ddq4,z0);
dv44 = dv(R34,w44,dw44,dv33,r34);
dv4c4 = dvc(dv44,w44,dw44,r4c);
'Forward recursion for link 4 completed'
 
% Link 5
w55 = w(R45,w44,dq4,z0);
dw55 = dw(R45,w44,dw44,dq5,ddq5,z0);
dv55 = dv(R45,w55,dw55,dv44,r45);
dv5c5 = dvc(dv55,w55,dw55,r5c);
'Forward recursion for link 5 completed'
 
% Link 6
w66 = w(R56,w55,dq5,z0);
dw66 = dw(R56,w55,dw55,dq6,ddq6,z0);
dv66 = dv(R56,w66,dw66,dv55,r56);
dv6c6 = dvc(dv66,w66,dw66,r6c);
'Forward recursion for link 6 completed'
 
%% Backward recursion
% Link 6
f66 = f(R67,f77,m6,R06,G,dv6c6);
n66 = n(R67,J6,f66,f77,r56,r6c,w66,dw66,n77);
tau6 = tau(R56,n66,z0);
'Backward recursion for link 6 completed'
 
% Link 5
f55 = f(R56,f66,m5,R05,G,dv5c5);
n55 = n(R56,J5,f55,f66,r45,r5c,w55,dw55,n66);
tau5 = tau(R45,n55,z0);
'Backward recursion for link 5 completed'
 
% Link 4
f44 = f(R45,f55,m4,R04,G,dv4c4);
n44 = n(R45,J4,f44,f55,r34,r4c,w44,dw44,n55);
tau4 = tau(R34,n44,z0);
'Backward recursion for link 4 completed'
 
% Link 3
f33 = f(R34,f44,m3,R03,G,dv3c3);
n33 = n(R34,J3,f33,f44,r23,r3c,w33,dw33,n44);
tau3 = tau(R23,n33,z0);
'Backward recursion for link 3 completed'
 
% Link 2
f22 = f(R23,f33,m2,R02,G,dv2c2);
n22 = n(R23,J2,f22,f33,r12,r2c,w22,dw22,n33);
tau2 = tau(R12,n22,z0);
'Backward recursion for link 2 completed'
 
% Link 1
f11 = f(R12,f22,m1,R01,G,dv1c1);
n11 = n(R12,J1,f11,f22,r01,r1c,w11,dw11,n22);
tau1 = tau(R01,n11,z0);
'Backward recursion for link 1 completed'
 
'Step 1 completed'

