%% MATLAB code: step2_compute_M_C_g.m

% Vectors
tau = [tau1;tau2;tau3];
q = [q1;q2;q3];
dq = [dq1;dq2;dq3];
ddq = [ddq1;ddq2;ddq3];
 
% Inertia matrix for Eq. (C.1)
mass = sym(zeros(3,3));
for i=1:3
   for j=1:3
      mass(i,j) = diff(tau(i),ddq(j));
   end
end
'Inertia matrix computed' 
 
% Symbolic substitution: Replace ci and si by cos(qi) and sin(qi) 
amass = sym(zeros(3,3));
for i = 1:3
   for j = 1:3
      amass(i,j) = subs(mass(i,j), s1, sin(q1));
      amass(i,j) = subs(amass(i,j), s2, sin(q2));
      amass(i,j) = subs(amass(i,j), s3, sin(q3));
        
      amass(i,j) = subs(amass(i,j), c1, cos(q1));
      amass(i,j) = subs(amass(i,j), c2, cos(q2));
      amass(i,j) = subs(amass(i,j), c3, cos(q3));
   end
end
 
% Coriolis matrix for Eq. (C.2)
coriolis = sym(zeros(3,3));
for i=1:3
   for j=1:2
      for k=1:3
         coriolis(i,j) = coriolis(i,j)+1/2*(diff(amass(i,j),q(k))+diff(amass(i,k),q(j))-diff(amass(j,k),q(i)))*dq(k);
      end
   end
end
 
% Symbolic substitution: Replace cos(qi) and sin(qi) by ci and si
for i = 1:3
   for j = 1:3
      coriolis(i,j) = subs(coriolis(i,j), sin(q1),s1);
      coriolis(i,j) = subs(coriolis(i,j), sin(q2),s2);
      coriolis(i,j) = subs(coriolis(i,j), sin(q3),s3);
        
      coriolis(i,j) = subs(coriolis(i,j), cos(q1),c1);
      coriolis(i,j) = subs(coriolis(i,j), cos(q2),c2);
      coriolis(i,j) = subs(coriolis(i,j), cos(q3),c3);    
  end
end
'Coriolis matrix computed'  
 
 % Gravity vector for Eq. (C.3)
for i=1:3
   gravity(i,1) = diff(tau(i),g)*g;
end
'Gravity vector computed' 
 
simplify(mass);
simplify(coriolis);
simplify(gravity);
 
'Step 2 completed'
